#include <iostream>
#include <vector>
#include <algorithm>

/** CHANGING STATE in the lambda --> capturing by reference
	NOTE : in this case no mutable needed

	Because C++ allows modifying referred to values in const member functions (const is shallow)
*/

void shorthand();

int main()
{
	int numVals = 0;
	int totalSoFar = 0;
	std::vector<double> averages;
	std::vector<int> vec = {101, 102, 2, 5, 11};

	std::transform(vec.cbegin(), vec.cend(), std::back_inserter(averages),
				[&](int i) {return static_cast<double>(totalSoFar +=i)/++numVals ;});


	std::for_each(averages.cbegin(), averages.cend(),
				[](double i) {std::cout << i << " ";});

	std::cout << std::endl << std::endl << std::endl;

    std::cout << numVals << " , " << totalSoFar << std::endl;

	shorthand();
    return 0;
}

void shorthand()
{
	std::vector<double> averages;
	std::vector<int> vec = {101, 102, 2, 5, 11};

	class MagicType1
	{
	public:
		MagicType1(int& v1, int& v2) :
			mNumVals(v1), mTotalSoFar(v2) {}
		double operator()(int i) const
		{
			return static_cast<double>(mTotalSoFar +=i)/++mNumVals;
		}
	private:
		int& mNumVals;
		int& mTotalSoFar;
	};

	int numVals = 0;
	int totalSoFar = 0;
	std::transform(vec.cbegin(), vec.cend(), std::back_inserter(averages),
					MagicType1(numVals, totalSoFar));

	std::for_each(averages.cbegin(), averages.cend(),
				[](double i) {std::cout << i << " ";});

	std::cout << std::endl << std::endl << std::endl;
}

